# Generated by roxygen2: do not edit by hand

S3method(as_rave_project,"ravecore::RAVEProjectImpl")
S3method(as_rave_project,RAVEProject)
S3method(as_rave_project,character)
S3method(as_rave_project,default)
S3method(close,glimpse_shinyapp)
S3method(collapse2,FileArray)
S3method(collapse2,RAVEFileArray)
S3method(collapse2,Tensor)
S3method(collapse2,array)
S3method(collapse_power,FileArray)
S3method(collapse_power,array)
S3method(plot,ravecore_volume_svg)
S3method(power_baseline,FileArray)
S3method(power_baseline,array)
S3method(power_baseline,rave_prepare_power)
S3method(print,RAVEValidation)
S3method(print,glimpse_shinyapp)
S3method(print,power_collapse_list)
S3method(print,ravecore_volume_svg)
export(Auxiliary_electrode)
export(IMPORT_FORMATS)
export(LFP_electrode)
export(LFP_reference)
export(LOCATION_TYPES)
export(MNI305_to_MNI152)
export(RAVEAbstarctElectrode)
export(RAVEEpoch)
export(RAVEPreprocessSettings)
export(RAVEProject)
export(RAVESubject)
export(RAVESubjectBaseRepository)
export(RAVESubjectEpochPhaseRepository)
export(RAVESubjectEpochPowerRepository)
export(RAVESubjectEpochRawVoltageRepository)
export(RAVESubjectEpochRepository)
export(RAVESubjectEpochTimeFreqCoefRepository)
export(RAVESubjectEpochVoltageRepository)
export(RAVESubjectRecordingBlockPhaseRepository)
export(RAVESubjectRecordingBlockPowerRepository)
export(RAVESubjectRecordingBlockRawVoltageRepository)
export(RAVESubjectRecordingBlockTimeFreqCoefRepository)
export(RAVESubjectRecordingBlockVoltageRepository)
export(SIGNAL_TYPES)
export(Spike_electrode)
export(YAELProcess)
export(YAEL_IMAGE_TYPES)
export(ants_coreg)
export(archive_subject)
export(as_rave_project)
export(as_rave_subject)
export(as_yael_process)
export(backup_file)
export(cache_root)
export(clear_cached_files)
export(cmd_afni_home)
export(cmd_dcm2niix)
export(cmd_dry_run)
export(cmd_execute)
export(cmd_freesurfer_home)
export(cmd_fsl_home)
export(cmd_homebrew)
export(cmd_run_3dAllineate)
export(cmd_run_ants_coreg)
export(cmd_run_dcm2niix)
export(cmd_run_freesurfer_recon_all)
export(cmd_run_freesurfer_recon_all_clinical)
export(cmd_run_fsl_flirt)
export(cmd_run_niftyreg_coreg)
export(cmd_run_nipy_coreg)
export(cmd_run_r)
export(cmd_run_yael_preprocess)
export(collapse2)
export(collapse_power)
export(compose_channel)
export(convert_electrode_table_to_bids)
export(export_table)
export(generate_atlases_from_template)
export(generate_reference)
export(get_available_morph_to_template)
export(get_projects)
export(glimpse_voltage_filearray)
export(glimpse_voltage_repository_with_blocks)
export(has_rave_subject)
export(import_from_brainvis)
export(import_from_edf)
export(import_from_h5_mat_per_block)
export(import_from_h5_mat_per_channel)
export(import_from_nevnsx)
export(import_table)
export(install_openneuro)
export(install_subject)
export(load_meta2)
export(new_electrode)
export(new_rave_subject)
export(new_reference)
export(niftyreg_coreg)
export(normalize_commandline_path)
export(plot_volume_slices)
export(power_baseline)
export(prepare_subject_bare)
export(prepare_subject_bare0)
export(prepare_subject_phase_with_blocks)
export(prepare_subject_phase_with_epochs)
export(prepare_subject_power)
export(prepare_subject_power_with_blocks)
export(prepare_subject_power_with_epochs)
export(prepare_subject_raw_voltage_with_blocks)
export(prepare_subject_raw_voltage_with_epochs)
export(prepare_subject_time_frequency_coefficients_with_blocks)
export(prepare_subject_time_frequency_coefficients_with_epochs)
export(prepare_subject_voltage_with_blocks)
export(prepare_subject_voltage_with_epochs)
export(prepare_subject_with_blocks)
export(prepare_subject_with_epochs)
export(py_nipy_coreg)
export(rave_brain)
export(rave_legacy_subject_format_conversion)
export(rave_path)
export(rscript_path)
export(run_wavelet)
export(save_meta2)
export(snapshot_project)
export(snapshot_subject)
export(transform_point_to_template)
export(transform_thinfilm_to_mni152)
export(validate_subject)
export(validate_time_window)
export(yael_preprocess)
importFrom(R6,R6Class)
importFrom(bidsr,BIDSProject)
importFrom(methods,getGeneric)
importFrom(methods,setMethod)
importFrom(ravepipeline,RAVESerializable)
